/*========================================================================

  File: dijkstraRev.c

  Program EPPSTEIN - Version 1.1 - July 1999

  ========================================================================

  This module contains an implementation of Bellman-Ford algorithm for
  computing the shortest path from every other node in a graph to the
  final node

  ========================================================================

    Copyright (C) 1999 Victor Jimenez and Andres Marzal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program (file COPYING); if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You can contact the authors at:

           Victor Jimenez and Andres Marzal
           Dept. de Informatica, Universitat Jaume I, Castellon, Spain
           {vjimenez,amarzal}@inf.uji.es

    Updated information about this software will be available at the
    following www address:

           http://terra.act.uji.es/REA
 
  ========================================================================= */

#include <stdlib.h>
#include <stdio.h>

#ifdef DEBUG
#include <assert.h>
#endif

#include <EPPSTEIN.h>

/*============================================================================*/
void DijkstraReversed (Graph *graph) {
  /* Computes the tree T with the best path from every node in the graph
     to the final node. Works for graphs with negative weigths.
     node->bestPath and node->bestCost was initialized by LoadGraph for every node.
  */
  Arc *arc;
  int i, j;

  graph->finalNode->bestCost = 0.0;
  graph->finalNode->bestPath = NULL;

  for(i=0; i<graph->numberNodes; i++){
	  for(j=0; j<graph->numberArcs; j++){
		  arc = graph->arc + j;
		  if( arc->cost + (arc->dest)->bestCost < (arc->source)->bestCost ){
			  (arc->source)->bestCost = arc->cost + (arc->dest)->bestCost;
			  (arc->source)->bestPath = arc->dest;
		  }
	  }
  }

  //check if there exists a negative cycle
/*  for(j=0; j<graph->numberArcs; j++){
    arc = graph->arc + j;
    if( arc->cost + (arc->dest)->bestCost < (arc->source)->bestCost ){
	  fprintf(stderr, "A negative cycle exists!\n");
	  exit(1);
	}
  }
*/

  return;
}
